#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <malloc.h>
#include <wiiuse/wpad.h>
#include <ogcsys.h>
#include <gccore.h>
#include <fat.h>

#include "../yui.h"

/* Wii includes */
#include "iowii.h"
#include "syswii.h"
#include "wuiwii.h"

/* Constants */
#define FILES_PER_PAGE	10
#define ROMS_DIRECTORY	"ROMS"

struct file *filelist = NULL;
u32 nb_files = 0;

s32 selected = 0, start = 0;

void Check_Bios_Exist(void)
{
	printf("\t[*] Check if bios.bin exist...");
	
	fatInitDefault();
	FILE *pFile = fopen(biosfilename,"r");
	if (pFile)
	{
		fclose(pFile);
		printf(" OK!\n");
	}
	else
	{
		printf(" ERROR! Return to Wii Menu in 5s...");
		sleep(5);
		Exit_ToWiiMenu();
	}
}

void Check_SD_Init(void)
{
	s32 ret;
	
	printf("\t[*] Initializing libfat...");
	fflush(stdout);
	ret = sd_init();
	if (!ret) {
		printf(" ERROR! Return to Wii Menu in 5s...");
		sleep(5);
		Exit_ToWiiMenu();
	}
	printf(" OK!\n");
}

void Check_Read_Dir(void)
{
	s32 ret;
	
	printf("\t[*] Reading SD card...");
	fflush(stdout);
	ret = sd_readdir(ROMS_DIRECTORY, &filelist, &nb_files);
	if (ret < 0) {
		printf(" ERROR! (ret = %d) Return to Wii Menu in 5s...", ret);
		sleep(5);
		Exit_ToWiiMenu();
	}
	printf(" OK!\n");
}

void manage_rom(struct file *finfo)
{
	/* Generate filepath */
	sprintf(isofilename, ROMS_DIRECTORY "/%s", finfo->filename);
	
	/* Execute Yabause */
	YuiExec();
}

void scroll_filelist(s8 delta)
{
	s32 index;

	/* No files */
	if (!nb_files)
		return;

	/* Select next entry */
	selected += delta;

	/* Out of the list? */
	if (selected <= -1)
		selected = nb_files - 1;
	if (selected >= nb_files)
		selected = 0;

	/* List scrolling */
	index = (selected - start);
	if (index >= FILES_PER_PAGE)
		start += (index - FILES_PER_PAGE) + 1;
	if (index <= -1)
		start += index;
}

void print_filelist(void)
{
	u32 cnt;

	printf("\t[*] Available files on the SD card:\n\n");

	/* No files */
	if (!nb_files) {
		printf("\t\t>> No files available!\n");
		return;
	}

	for (cnt = start; cnt < nb_files; cnt++) {
		/* Files per page limit */
		if ((cnt - start) >= FILES_PER_PAGE)
			break;

		/* Selected file */
		if (cnt == selected)
			printf("\t\t>> ");
		else
			printf("\t\t   ");
		fflush(stdout);

		/* Print filename */
		printf("%s\n", filelist[cnt].filename);
	}
}

void controls(void)
{	
	WPAD_ScanPads();

	/* UP button */
	if (WPAD_ButtonsDown(0) & WPAD_CLASSIC_BUTTON_UP ||
		WPAD_ButtonsDown(0) & WPAD_BUTTON_UP)
		scroll_filelist(-1);

	/* DOWN button */
	if (WPAD_ButtonsDown(0) & WPAD_CLASSIC_BUTTON_DOWN ||
		WPAD_ButtonsDown(0) & WPAD_BUTTON_DOWN)
		scroll_filelist(1);

	/* 1 button - X button */
	if (WPAD_ButtonsDown(0) & WPAD_CLASSIC_BUTTON_X ||
		WPAD_ButtonsDown(0) & WPAD_BUTTON_1)
		Exit_ToHBCMenu();
	
	/* 2 button - Y button */
	if (WPAD_ButtonsDown(0) & WPAD_CLASSIC_BUTTON_Y ||
		WPAD_ButtonsDown(0) & WPAD_BUTTON_2)
		Exit_ToWiiMenu();

	/* A button */
	if (WPAD_ButtonsDown(0) & WPAD_CLASSIC_BUTTON_A ||
		WPAD_ButtonsDown(0) & WPAD_BUTTON_A)
		manage_rom(&filelist[selected]);
}
