/*
	v0.9.6 Original release by Yabuse Team
	First compilation by ds342
	First edition by Markus1981

	Second edition by Ac_K :
		- ThX to Waninkoko for WAD Manager source
		- ISO selection menu added with help of WAD Manager source
		- Useless functions remove
		- Littles code optimisations
		- Check ISO extension
		- Check if BIOS exist
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <malloc.h>
#include <wiiuse/wpad.h>
#include <ogcsys.h>
#include <gccore.h>
#include <fat.h>

/* Yabause includes */
#include "../cs0.h"
#include "../m68kcore.h"
#include "../peripheral.h"
#include "../vidsoft.h"
#include "../vdp2.h"
#include "../yui.h"

/* Wii includes */
#include "perwii.h"
#include "sndwii.h"
#include "iowii.h"
#include "syswii.h"
#include "wuiwii.h"

//////////////////////////////////////////////////////////////////////////////

SH2Interface_struct *SH2CoreList[] = {
	&SH2Interpreter,
	&SH2DebugInterpreter,
	NULL
};

PerInterface_struct *PERCoreList[] = {
	&PERDummy,
	&PERWiiKeyboard,
	&PERWiiClassic,
	NULL
};

CDInterface *CDCoreList[] = {
	&DummyCD,
	&ISOCD,
	NULL
};

SoundInterface_struct *SNDCoreList[] = {
	&SNDDummy,
	&SNDWII,
	NULL
};

VideoInterface_struct *VIDCoreList[] = {
	&VIDDummy,
	&VIDSoft,
	NULL
};

M68K_struct *M68KCoreList[] = {
	&M68KDummy,
	&M68KC68K,
	NULL
};

//////////////////////////////////////////////////////////////////////////////

/* Global variables */

//static char bupfilename[512] = "bkram.bin";

extern int vdp2width, vdp2height;

static u32 *xfb[2] = { NULL, NULL };
int fbsel = 0;
static GXRModeObj *rmode = NULL;
volatile int done = 0;
volatile int resetemu = 0;
int running = 1;
static void *_console_buffer = NULL;
void __console_init_ex(void *conbuffer,int tgt_xstart,int tgt_ystart,int tgt_stride,int con_xres,int con_yres,int con_stride);

//////////////////////////////////////////////////////////////////////////////

void gotoxy(int x, int y);
void OnScreenDebugMessage(char *string, ...);
void DoMenu();

//////////////////////////////////////////////////////////////////////////////

void reset()
{
	resetemu=1;
}

void powerdown()
{
	done = 1;
}

int main(int argc, char **argv)
{
	WPAD_Init();
	SYS_SetResetCallback(reset);
	SYS_SetPowerCallback(powerdown);
	DVDStopMotor();
	
	DoMenu();

	return 0;
}

int YuiExec()
{
	yabauseinit_struct yinit;
	int ret;

	VIDEO_Init();

	switch(VIDEO_GetCurrentTvMode()) 
	{
		case VI_NTSC:
			rmode = &TVNtsc240Ds;
			break;
		case VI_PAL:
			rmode = &TVPal264Ds;
			break;
		case VI_MPAL:
			rmode = &TVMpal480IntDf;
			break;
		default:
			rmode = &TVNtsc240Ds;
			break;
	}
	
	// Allocate two buffers(may not be necessary)
	xfb[0] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	xfb[1] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	
	VIDEO_Configure(rmode);
	VIDEO_ClearFrameBuffer (rmode, xfb[0], COLOR_BLACK);
	VIDEO_ClearFrameBuffer (rmode, xfb[1], COLOR_BLACK);
	VIDEO_SetNextFramebuffer(xfb[0]);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) 
		VIDEO_WaitVSync();

	free(_console_buffer);
	_console_buffer = malloc(10*10*VI_DISPLAY_PIX_SZ);
	__console_init_ex(_console_buffer,10,10,rmode->fbWidth*VI_DISPLAY_PIX_SZ,10,10,10*VI_DISPLAY_PIX_SZ);

	memset(&yinit, 0, sizeof(yabauseinit_struct));
	yinit.percoretype = PERCORE_WIICLASSIC;
	yinit.sh2coretype = SH2CORE_INTERPRETER;
	yinit.vidcoretype = VIDCORE_SOFT;
	yinit.sndcoretype = SNDCORE_WII;
	yinit.cdcoretype = CDCORE_ISO;
	yinit.m68kcoretype = M68KCORE_C68K;
	yinit.carttype = CART_NONE;
	yinit.regionid = REGION_AUTODETECT;
	if (strcmp(biosfilename, "") == 0)
		yinit.biospath = NULL;
	else
		yinit.biospath = biosfilename;
	yinit.cdpath = isofilename;
	//yinit.buppath = bupfilename;
	yinit.buppath = NULL;
	yinit.mpegpath = NULL;
	yinit.cartpath = NULL;
	yinit.netlinksetting = NULL;
	yinit.flags = VIDEOFORMATTYPE_NTSC;

	// Hijack the fps display
	VIDSoft.OnScreenDebugMessage = OnScreenDebugMessage;

	if ((ret = YabauseInit(&yinit)) == 0)
	{
		EnableAutoFrameSkip();
		VIDEO_ClearFrameBuffer(rmode, xfb[fbsel], COLOR_BLACK);

		while(!done)
		{
			if (PERCore->HandleEvents() != 0)
				return -1;
			if (resetemu)
			{
				YabauseReset();
				resetemu = 0;
				SYS_SetResetCallback(reset);
			}
		}
		YabauseDeInit();
	}
	else
	{
		while(!done)
			VIDEO_WaitVSync();
	}

	return 0;
}

void YuiErrorMsg(const char *string)
{
	if (strncmp(string, "Master SH2 invalid opcode", 25) == 0)
	{
		if (!running)
			return;
		running = 0;
		printf("%s\n", string);
	}
}

void YuiSwapBuffers()
{
	int i, j;
	u32 *curfb;
	u32 *buf;

	fbsel ^= 1;
	curfb = xfb[fbsel];
	buf = dispbuffer;

	for (j = 0; j < vdp2height; j++)
	{
		for (i = 0; i < vdp2width; i++)
		{
			// This isn't pretty
			int y1, cb1, cr1;
			int cb, cr;
			u8 r, g, b;
      
			 r = buf[0] >> 24;
			 g = buf[0] >> 16;
			 b = buf[0] >> 8;
			 buf++;
			
			 y1 = (299 * r + 587 * g + 114 * b) / 1000;
			 cb1 = (-16874 * r - 33126 * g + 50000 * b + 12800000) / 100000;
			 cr1 = (50000 * r - 41869 * g - 8131 * b + 12800000) / 100000;
			 cb = (cb1 + cb1) >> 1;
			 cr = (cr1 + cr1) >> 1;

			 curfb[0] = (y1 << 24) | (cb << 16) | (y1 << 8) | cr;
			 curfb++;
		}
	}

	VIDEO_SetNextFramebuffer (xfb[fbsel]);
	VIDEO_Flush ();
}

void gotoxy(int x, int y)
{
	printf("\033[%d;%dH", y, x);
}

void OnScreenDebugMessage(char *string, ...)
{
	va_list arglist;

	va_start(arglist, string);
	gotoxy(0, 1);
	vprintf(string, arglist);
	printf("\n");
	gotoxy(0, 1);
	va_end(arglist);
}

void DoMenu()
{	
	
	VIDEO_Init();
	switch(VIDEO_GetCurrentTvMode()) 
	{
		case VI_NTSC:
			rmode = &TVNtsc480IntDf;
			break;
		case VI_PAL:
			rmode = &TVPal528IntDf;
			break;
		case VI_MPAL:
			rmode = &TVMpal480IntDf;
			break;
		default:
			rmode = &TVNtsc480IntDf;
			break;
	}

	xfb[0] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	xfb[1] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	VIDEO_Configure(rmode);
	VIDEO_ClearFrameBuffer (rmode, xfb[0], COLOR_BLACK);
	VIDEO_ClearFrameBuffer (rmode, xfb[1], COLOR_BLACK);
	VIDEO_SetNextFramebuffer(xfb[0]);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) 
		VIDEO_WaitVSync();
	free(_console_buffer);
	_console_buffer = malloc(600*600*VI_DISPLAY_PIX_SZ);
	__console_init_ex(_console_buffer,20,20,rmode->fbWidth*VI_DISPLAY_PIX_SZ,600,600,600*VI_DISPLAY_PIX_SZ);

	printf("\x1b[2;0H"); // Put text at line 2

	Check_SD_Init(); // Initialize SD subsystem	
	Check_Read_Dir(); // Read ROMS directory
	Check_Bios_Exist(); // Check if bios.bin exist
	
	
	sleep(3); // Wait 3s
	
	while(1)
	{
		VIDEO_WaitVSync();

		printf("\x1b[2J\n\n"); // Clear screen
		printf("\x1b[2;0H"); // Put text at line 2
		
		printf("\t+----------------------------+\n");
		printf("\t|         Yabuse Wii v0.9.6  |\n");
		printf("\t|  developed by Yabuse Team  |\n");
		printf("\t|          modified by Ac_K  |\n");
		printf("\t+----------------------------+\n");
		printf("\t|      ThX to Waninkoko for  |\n");
		printf("\t|    WADManager source code  |\n");
		printf("\t+----------------------------+\n");
		printf("\t+-- Menu Controls ----------------------------------------+\n");
		printf("\t| (A) Load ROM | (1) Load HBC Menu | (2) Load System Menu |\n");
		printf("\t+---------------------------------------------------------+\n\n");
		
		print_filelist();
		
		controls();
	}
}
